/* ========================================================================
   MULTICOMMODITY (CONCURRENT) FLOW PROGRAM

   Uses the following files:
       MULTICOM3.C	main program
       MULTICOMIO.C	input / output routines
       ROUTE.C		routine to find initial routes
       DECONGEST3.C	iterated procedure
       MINCOST.C	minimum-cost flow routine (Bertsekas & Tseng)
       COMMON.H		include files; macro, type, struct definitions
       MCF.H		global variable and array declarations
       EXPINTEGERS	integral powers of e
       EXPHUNDRETHS	decimal powers of e (to the 2nd decimal place)
       LIBF2C.A		library functions from Fortran to C conversion

   Written:
       August 8, 1991 by Tishya A. Leong

   Usage:
       mcf3 inputfile outputfile
   ======================================================================== */


/* ------------------------------------------------------------------------
   File: COMMON.H
   Purpose: Include file for standard include libraries, macro definitions, 
	    type definitions, and structure definitions
   ------------------------------------------------------------------------ */


/* ------------------------------------------------------------------------
   Include libraries
   ------------------------------------------------------------------------ */
#include <stdio.h>
#include <math.h>
#include <time.h>


/* ------------------------------------------------------------------------
   Macro definitions
   ------------------------------------------------------------------------ */
#define NULL 0
#define MIN(X, Y) ((X < Y) ? X : Y)
#define MAX(X, Y) ((X > Y) ? X : Y)


/* ------------------------------------------------------------------------
   Type definitions
   ------------------------------------------------------------------------ */
typedef double Ftype;


/* ------------------------------------------------------------------------
   Structure definitions
   ------------------------------------------------------------------------ */
struct groupType {	    /* specifies a commodity group */
	   int source;
	   int *demands;
       };

struct cost_list_elt {	    /* element in list for ordering groups by cost */
	   Ftype cost;		
	   Ftype minCost;
	   int commodity;
       };
